package ActiveState::DB;

use strict;

our $VERSION = '6.000';

use XSLoader ();
XSLoader::load(__PACKAGE__, $VERSION);

package ActiveState::DB::Manager;

use constant MASTER => 0;
use constant NESTED => 1;

sub new {
    my ($cls, $db) = @_;
    bless [$db, {}], ref($cls) || $cls;
}

sub DESTROY { 
    my $self = shift;
    $self->destroy if @$self;
}

sub destroy {
    my $self = shift;
    for my $nest (keys %{$self->[NESTED]}) {
	my $db = delete $self->[NESTED]{$nest};
	$db->destroy if $db;
    }
    $self->[MASTER]->destroy if $self->[MASTER];
    @$self = ();
}

sub nest {
    my ($self, $key) = @_;
    my $master = $self->[MASTER];
    my $nested = $self->[NESTED];
    return $nested->{$key} ||= ActiveState::DB::Nested->new($master, $key);
}

sub nest_manager {
    my ($self, $key) = @_;
    my $master = $self->[MASTER];
    my $nested = $self->[NESTED];
    return $nested->{$key} ||= do {
	my $db = ActiveState::DB::Nested->new($master, $key);
	$self->new($db);
    };
}

sub nest_object {
    my ($self, $key, $class) = @_;
    my $master = $self->[MASTER];
    my $nested = $self->[NESTED];
    return $nested->{$key} ||= $class->new($master, $key);
}

sub master {
    my $self = shift;
    $self->[MASTER];
}

sub nested {
    my ($self, $key) = @_;
    $self->[NESTED]{$key};
}

1;

=head1 NAME

ActiveState::DB - lightweight versatile database for in-memory or mmap'd files

=head1 SYNOPSIS

 use ActiveState::DB;

 # From a read-only file
 $db = ActiveState::DB->new("file");

 # From a string (copies the string)
 $db = ActiveState::DB::String->new("string");

 # Nested databases
 $manager = ActiveState::DB::Manager->new($db);
 $keydb   = $manager->nest("key");
 $key2db  = $manager->nest("key2");

 # Write your own database!
 $wdb = ActiveState::DB::Build->new;
 $wdb->set(key => value);
 $wdb->set(key2 => value2);
 ...
 $wdb->build_to("filename");
 $string = $wdb->build;

=head1 DESCRIPTION

The main goal behind ActiveState::DB is to put read-only data in a shared
memory region to cut down on overall memory usage. It is slower than using a
Perl hash because of the method call overhead Perl imposes.

For those applications where speed is more important than memory,
ActiveState::DB may not be the right solution at all.

=head1 ActiveState::DB API

The base class C<ActiveState::DB> provides a simple API, plus read-only access
to ActiveState::DB files.

=over

=item new()

 $db = ActiveState::DB->new("file");

Creates a new ActiveState::DB object that reads its data from C<file>, which
is opened and memory mapped readonly. Dies if anything goes wrong.

The behaviour is not defined if the underlying file changes while it is
opened. The application is responsible for protecting the file.
ActiveState::DB::Build helps with this by writing to a temporary file and
renaming it into place on systems where this is supported.

=back

The following methods are common to all ActiveState::DB objects, no matter
what their storage format:

=over

=item get()

 $val = $db->get("key");

Fetch a value from the database. Returns the value associated with C<key>, or
C<undef> if the key is not in the database.

=item exists()

 $exists = $db->exists("key");

Returns C<1> if the C<key> exists, otherwise C<0>.

=item enumerate()

 $count = $db->enumerate(\&callback, $data);

Invokes the C<callback> once for each key/value pair in the database,
returning the number of times C<callback> was invoked. The callback should
look like this:

 sub callback {
     my ($data, $key, $value) = @_;
     ...
     return 1;
 }

Enumeration stops when the callback returns false, or after the last key/value
pair.

If the C<callback> is C<undef>, C<enumerate()> cheaply returns the number of
key/value pairs in the database.

I<NOTE>: enumerate guarantees to return sorted keys, such that short keys come
before long keys, and keys of equal length are lexicographically sorted.

=back

The following methods are provided so that other extensions written in XS can
use ActiveState::DB objects using the C API.

=over

=item _get()

 ($ptr, $len) = $db->_get($key);

Looks up C<$key> in the database, and returns the value as a C pointer and
length. Behaviour is not defined if $ptr is written to. The application must
ensure that the lifetime of $ptr does not exceed $db.

=item _asdb()

 $ptr = $db->_asdb;

Returns a C pointer to the underlying ASDB object. The C<$db> object owns the
C pointer, so don't try to free it.

=item _new()

 $ptr = $db->_new($raw, $size)

Returns a C pointer to a newly-created ASDB object that refers to the memory
region specified by C<$raw> and C<$size>. If the C<$ptr> is 0, creation
failed; otherwise the newly created object belongs to the caller.

=back

=head1 ActiveState::DB::Manager API

ActiveState::DB::Manager is a wrapper around ActiveState::DB that manages
nested ActiveState::DB objects. It keeps track of the master table and all
nested databases, and arranges to DESTROY the children when the manager is
destroyed.

=over

=item new()

 $dbm = ActiveState::DB::Manager->new($master);

Creates a new manager around C<$master>. The C<$master> is now owned by the
manager, and should not be saved by the application.

=item nest()

 $db = $dbm->nest("key");

Creates a nested table that refers to the C<$master>. The new table holds a
reference to C<$master>.

=item nest_manager()

 $dbm2 = $dbm->nest_manager("key");

Creates a nested table that refers to the C<$master>, then wraps it in a new
C<ActiveState::DB::Manager> and stores that in C<$dbm> instead. This allows
you to nest tables to arbitrary depth.

=item nest_object()

 $obj = $dbm->nest_object("key", "CLASS");

Creates a new C<CLASS> object, passing it the C<master> database and the
C<$key>. The object is returned, and also cached in the engine: multiple
requests for the same C<$key> will return the same object, regardless of
whether the C<CLASS> changes.

The C<CLASS> must support a C<destroy()> method; the manager calls this method
when it is destroyed. The object I<MUST> forget all cached references to the
database, since it will be invalid from that point on.

=item destroy()

 $dbm->destroy;

Destroys all nested databases, managers, and objects, then the $master
database. Any saved objects returned from nest(), nest_manager(), or
nest_object() will be invalid. This method is automatically when $dbm goes out
of scope.

=item master()

 $master = $dbm->master;

Returns the C<$master> passed to the constructor.

=back

=head1 ActiveState::DB::String API

It can be very useful to pass ActiveState::DB objects around as a binary
string. One caveat is that the constructor copies the string, so be careful
not to use this for very large databases.

=over

=item new()

 $db = ActiveState::DB::String->new("string");

Copies C<string> and uses it as a database. Dies if this is not a valid
ActiveState::DB.

=back

=head1 ActiveState::DB::Build API

ActiveState::DB files are only for reading, so you must write a new file to
apply modifications.

=over

=item new()

 $bld = ActiveState::DB::Build->new;

Creates a new ActiveState::DB::Build object to store the keys and values until
they can be flushed to disk.

=item set()

 $bld->set("key", "val");

Adds a key/value pair.

I<NOTE>: if the same C<key> is set multiple times, which value will be saved
is unspecified. Internally ActiveState::DB might use an unstable sorting
algorithm, which means it doesn't know which key was specified first. If this
is important, use a C<%seen> hash as you insert values into the database to
avoid inserting duplicates.

=item _set()

 $bld->_set("key", $val_ptr, $val_len);

Adds a key/value pair.  The value is decoded as a C pointer and length.  The
memory at C<$val_ptr> is copied into the new database during this method.
Application behaviour is undefined if the memory referenced by C<$val_ptr> and
C<$val_len> is not readable by the application.

=item index()

 $bld->index($type);

Specifies the type of index to use. Must be one of the following strings:

=over

=item HASH

Use a hash lookup table. This yields the fastest lookups.

=item BIDX

Use a binary search table. This takes less space than a C<HASH> table, but is
slower.

=item NONE

Don't save an index. This takes even less memory than a C<BIDX>. Use this if
you only need to call enumerate() (get() uses a linear search, which is very
slow).

=back

=item build()

 $string = $bld->build;

Builds a database image and returns it. The string can be saved to a file, or
opened with C<ActiveState::DB::String>.

=item build_to()

 $bld->build_to("filename");

Atomically creates a database image and saves it to C<filename>.

=back
